<?php
// Original Markdown System
//
// 2023/01/27 目次、参照対応
// 2023/01/29 クリップボード機能、YouTube参照機能
// 2023/01/30 下線、マーカー
// 2023/02/09 アンカー埋め込み、アンカーリンク
//
// 2023 (c) Crytus Y.Kurihara
//
define("VER", "0.98");

mb_regex_encoding('UTF-8');

static $refer = array();	// リンクなどの参照
static $macro = array();	// マクロ定義
static $text_block = array();	// テキストブロック
static $pre;
static $contents = "";
static $post = "";
static $reference = array();	// 参照文献など
//static $onerror = " onError='this.src=\"noimage.png\"'";	// 画像がなかった場合
static $clipboard = 1;	// クリップボードカウンター

// 文字列をマークダウン処理する。（行に分解して処理する）
function markdown($str)
{
	global $pre, $contents, $post;
	global $clipboard;

	$contents = "";
	$post = "";
	//
	$pre = "<!-- Crytus Markdown Ver. " . VER . " (" . date('Ymd', filemtime(__FILE__)) . ") -->\n";
	$lines = array_map("trim_le", explode("\n", $str));
	pre_translate($lines);
	md_proc($lines);
	if ($clipboard > 1) {
	$post .= "<script>\n";
	$post .= "function clipboard(id) {\n";
	$post .= "    var target = document.getElementById(id);\n";
	$post .= "    target.select();\n";
	$post .= "    document.execCommand(\"Copy\");\n";
	$post .= "    alert(\"コピーしました。\");\n";
	$post .= "}\n";
	$post .= "</script>\n";
	}
	$html = $pre . $contents . $post;
	return $html;
}
// 行末の改行コードを取り去る
function trim_le($str)
{
	return trim($str, "\r\n");
}
// 拡張タグのチェック
function tagCheck($tag, $line)
{
	$str = null;
	if (substr(trim($line), 0, strlen($tag)) == $tag) {
		$str = trim(substr($line, strlen($tag)));
	}
	return $str;
}
// マクロ展開
function macro($line)
{
	global $macro;

	foreach ($macro as $key => $val) {
		while (mb_eregi("\{\%" . $key . "([^\%]+)\%\}", $line, $m)) {
			$ary = explode("|", $m[1]);
			if ($ary) {
				while (mb_eregi("\%([0-9]+)", $val, $mm)) {
					$val = str_replace($mm[0], $ary[$mm[1]], $val);
				}
			}
			$line = str_replace($m[0], $val, $line);
		}
	}
	return $line;
}
// インライン展開
function inline($line)
{
	global $text_block;
	global $refer;
	global $reference;
	global $onerror;

	// <画像のパス>|<サイズw>|<サイズh>|<alt>|<説明>
	while (mb_eregi("\{\%img ([^\%]+)\%\}", $line, $m)) {
		$ary = explode("|", $m[1]);
		if ($ary) {
			$txt = "";
			if ($ary[4]) {
				$txt .= "<figure>";
			}
			$txt .= "<img class=\"img-fluid\" src=\"" . $ary[0] . "\"";
			if ($ary[1] != "") {
				$txt .= " width=\"{$ary[1]}\"";
			}
			if ($ary[2] != "") {
				$txt .= " height=\"{$ary[2]}\"";
			}
			if ($ary[3] != "") {
				$txt .= " alt=\"{$ary[3]}\"";
			}
			$txt .= "{$onerror}>";
			if ($ary[4]) {
				$txt .= "<figcaption>{$ary[4]}</figcaption>";
				$txt .= "</figure>";
			}
		}
		$line = str_replace($m[0], $txt, $line);
	}
	//  <アイコン番号> <サイズなど>
	while (mb_eregi("\{\%icon ([^\%]+)\%\}", $line, $m)) {
		$ary = explode("|", $m[1]);
		$txt = "";
		if ($ary) {
			$txt = "<i class=\"{$ary[0]}\" rold=\"img\" style=\"{$ary[1]}\"></i>";
		}
		$line = str_replace($m[0], $txt, $line);
	}
	//  <バッヂの種類> <内容>
	while (mb_eregi("\{\%badge ([^\%]+)\%\}", $line, $m)) {
		$ary = explode("|", $m[1]);
		$txt = "";
		if ($ary) {
			$txt .= "<span class=\"badge {$ary[1]}\">{$ary[0]}</span>";
		}
		$line = str_replace($m[0], $txt, $line);
	}
	while (mb_eregi("\{\%button ([^\%]+)\%\}", $line, $m)) {
		$ary = explode("|", $m[1]);
		$txt = "";
		if ($ary) {
			$txt .= "<button type=\"{$ary[0]}\" class=\"btn {$ary[1]}\" onClick=\"{$ary[3]}\">";
			$txt .= $ary[2];
			$txt .= "</button>";
		}
		$line = str_replace($m[0], $txt, $line);
	}
	// ブロックテキスト
	while (mb_eregi("\{\%text ([^\%]+)\%\}", $line, $m)) {
		$ary = explode("|", $m[1]);
		$txt = "";
		if ($ary) {
			//$txt = $text_block[trim($ary[0])];
			$txt = inline(macro(trim($text_block[trim($ary[0])])));
		}
		$line = str_replace($m[0], $txt, $line);
	}
	// {%input <type>|<名前>|<値>%}
	while (mb_eregi("\{\%input ([^\%]+)\%\}", $line, $m)) {
		$ary = explode("|", $m[1]);
		$txt = "";
		if ($ary) {
			$txt = "<input type=\"{$ary[0]}\" name=\"{$ary[1]}\" value=\"{$ary[2]}\"/>";
		}
		$line = str_replace($m[0], $txt, $line);
	}
	// 参照 %%語句|参照元%%
	while (mb_eregi("\{\%ref([^\|]+)\|([^\%]+)\%\}", $line, $m)) {
		// 同じものがあればまとめる
		foreach ($reference as $k => $v) {
			if ($v == $m[2]) {
				break;
			}
		}
		if ($v == $m[2]) {
			$txt = "<cite>" . $m[1] . "</cite><sup>※{$k}</sup>"; 
		} else {
			$n = count($reference) + 1;
			$reference[$n] = $m[2];
			$txt = "<cite>" . $m[1] . "</cite><sup>※{$n}</sup>"; 
		}
		$line = str_replace($m[0], $txt, $line);
	}
	// Markdown
	// ___abc[class]___ underline
	while (mb_eregi("___([^\_]*)?___", $line, $m)) {
		if (mb_eregi("\[(.*)\]$", $m[1], $m2)) {
			$str = str_replace($m2[0], "", $m[1]);
			$txt = "<span class=\"{$m2[1]}\">{$str}</span>";
			$line = str_replace($m[0], $txt, $line);
		} else {
			$txt = "<u>{$m[1]}</u>";
			$line = str_replace($m[0], $txt, $line);
		}
	}
/*
	// ___abc___ underline
	while (mb_eregi("___([^_]+)___", $line, $m)) {
		$txt = "<u>{$m[1]}</u>";
		$line = str_replace($m[0], $txt, $line);
	}
*/
	// **abc** bold
	while (mb_eregi("\*\*([^\*]+)\*\*", $line, $m)) {
		$txt = "<strong>{$m[1]}</strong>";
		$line = str_replace($m[0], $txt, $line);
	}
	// *abc* italic
	while (mb_eregi("\*([^\*]+)\*", $line, $m)) {
		$txt = "<em>{$m[1]}</em>";
		$line = str_replace($m[0], $txt, $line);
	}
	// __abc__ bold
	while (mb_eregi("__([^_]+)__", $line, $m)) {
		$txt = "<strong>{$m[1]}</strong>";
		$line = str_replace($m[0], $txt, $line);
	}
/* URLが壊れる　https://test_123/123_test/
	// _abc_ italic
	while (mb_eregi("_([^_]+)_", $line, $m)) {
		$txt = "<em>{$m[1]}</em>";
		$line = str_replace($m[0], $txt, $line);
	}
*/
	// ~~abc~~ 取り消し
	while (mb_eregi("\~\~([^\~]+)\~\~", $line, $m)) {
		$txt = "<strike>{$m[1]}</strike>";
		$line = str_replace($m[0], $txt, $line);
	}
	// `abc` code
	while (mb_eregi("\`([^\`]+)\`", $line, $m)) {
		$v = htmlspecialchars($m[1]);
		$txt = "<code>{$v}</code>";
		$line = str_replace($m[0], $txt, $line);
	}
	// ![]() 画像
	while (mb_eregi("\!\[([^\]]+)\]\(([^\)]+)\)", $line, $m)) {
		$txt = "<img src=\"{$m[2]}\" alt=\"{$m[1]}\"{$onerror}>";
		$line = str_replace($m[0], $txt, $line);
	}
	// ![][] 画像参照
	while (mb_eregi("\!\[([^\]]+)\]\[([^\)]+)\]", $line, $m)) {
		$v = $refer[strtolower($m[2])];
		$txt = "<img src=\"{$v}\" alt=\"{$m[1]}\"{$onerror}>";
		$line = str_replace($m[0], $txt, $line);
	}
	// []() リンク
	while (mb_eregi("\[([^\]]+)\]\(([^\)]+)\)", $line, $m)) {
		$txt = "<a href=\"{$m[2]}\">{$m[1]}</a>";
		$line = str_replace($m[0], $txt, $line);
	}
	// [You can use numbers for reference-style link definitions][1]　参照
	while (mb_eregi("\[([^\]]+)\]\[([^\)]+)\]", $line, $m)) {
		$v = $refer[strtolower($m[2])];
		$txt .= "<a href=\"{$v}\">{$m[1]}</a>";
		$line = str_replace($m[0], $txt, $line);
	}
	// [...] -> <a href="{...}">...</a>
	if (mb_eregi("\[([^\]]+)\]", $line, $m)) {
		if ($refer[strtolower($m[1])]) {
			$v = $refer[strtolower($m[1])];
			$txt .= "<a href=\"{$v}\">{$m[1]}</a>";
			$line = str_replace($m[0], $txt, $line);
		} else {
		//	$line = str_replace($m[0], "", $line);
		}
	}
	// url link
	$line = mb_ereg_replace("\<(http|https)(\:\/\/[\w\.\-&%$#!?]+[\:0-9+]*[0-9a-zA-Z\.\~\/\-?&=_#\:%;]*)\>", "<a href=\"\\1\\2\" target=\"_brank\">\\1\\2</a>", $line);
	return $line;
}
// TEXTとMACROと参照の事前検出、登録、目次の作成
function pre_translate($lines)
{
	global $macro, $refer;
	global $text_block;
	static $block = 0;	// ブロック要素処理
	global $idx;
	global $idx_no;

	$block_name = "";
	$block_text = "";

//	foreach ($lines as $line) {
	for ($lno = 0; $lno < count($lines); $lno++) {
		$line = $lines[$lno];
		if ($block == 1) {
			if (($s = tagCheck("%%TEXTEND",$line)) !== null) {
				$block = 0;
				$text_block[$block_name] = $block_text;
			} else {
				$block_text .= trim($line) . "\n";
			}
			continue;
		}
		// ```の間をスキップ
		if (mb_eregi("^```", $line, $m)) {
			$cnt = 0;
			for ($i = $lno + 1; $i < count($lines); $i++) {
				if (mb_eregi("^```", $lines[$i], $m)) {
					break;
				}
				$cnt++;
			}
			if ($cnt) {		// 終わりが存在
				$lno = $i + 1;
				continue;
			}
		}
		// ~~~の間をスキップ
		if (mb_eregi("^~~~", $line, $m)) {
			$cnt = 0;
			for ($i = $lno + 1; $i < count($lines); $i++) {
				if (mb_eregi("^~~~", $lines[$i], $m)) {
					break;
				}
				$cnt++;
			}
			if ($cnt) {		// 終わりが存在
				$lno = $i + 1;
				continue;
			}
		}
		if (($s = tagCheck("%%TEXT ",$line)) !== null) {
			$block = 1;
			$block_text = "";
			$ary = explode("|", trim($s));
			$block_name = $ary[0];
		}
		if (($s = tagCheck("%%MACRO ",$line)) !== null) {
			// <名前> <展開内容>
			//%%<マクロ名前>|<オプション>
			$ary = explode("|", trim($s));
			if ($ary) {
				$macro[array_shift($ary)] = implode("|", $ary);
			}
		}
		// includeファイルの中も解析しておく
		if (($s = tagCheck("%%INCLUDE ",$line)) !== null) {
			if ($s and is_readable($s)) {
				$filecontents = file_get_contents($s);
				$ln = array_map('trim', explode("\n", $filecontents));
				pre_translate($ln);
			}
		}
		// 目次作成（この場所に目次を埋め込む）
		if (($s = tagCheck("%%TOC ",$line)) !== null) {
			$idx = array();
			$idx_no = 1;
		}
		// 目次解析
		// # ## ### #### ##### #####を対象とする
		if (mb_eregi("^[ 	]*(\#+) (.*)$", $line, $m)) {
			$level = strlen($m[1]);
			unset($i);
			$i["level"] = $level;
			$i["title"] = $m[2];
			$i["id"] = $idx_no++;
			$idx[] = $i;
		}
		// markdown reference
		// [arbitrary case-insensitive reference text]: https://www.mozilla.org [1]: http://slashdot.org [link text itself]: http://www.reddit.com
		// [ ]:
		while (mb_eregi("\[([^\]]*)\]: ([^ ]+)", $line, $m)) {
			$refer[strtolower($m[1])] = $m[2];
			$line = str_replace($m[0], "", $line);
		}
	}
}
// 表解析
function table($line)
{
	$c = 0;
	$ary = array();
	while (mb_eregi("\|([^|]+)\|", $line, $m)) {
		$ary[$c] = $m[1];
		$c++;
		$line = substr($line, strlen($m[0]) - 1);
	}
	return array($c, $ary);
}
// ファイル読み込み処理
function md_proc($lines)
{
	global $pre, $contents, $post;
	global $title, $desc;
	global $macro;
	static $block = 0;	// ブロック要素処理
	static $para = 0;	// パラグラフ
	global $idx;
	global $idx_no;
	global $reference;
	global $onerror;
	global $clipboard;

	for ($lno = 0; $lno < count($lines); $lno++) {
//		$line = trim($lines[$lno], "\n\r");
		$line = $lines[$lno];
		if (substr($line, 0, 2) == "%%") {
			if ($block == 1) {
				if (($s = tagCheck("%%TEXTEND",$line)) !== null) {
					$block = 0;
				}
				continue;
			}
			if (($s = tagCheck("%%TEXT ",$line)) !== null) {
				$block = 1;
			}
			if (($s = tagCheck("%% ",$line)) !== null) {
				// エスケープ
				$contents .= htmlspecialchars($s);
			}
			if (($s = tagCheck("%%TITLE ",$line)) !== null) {
				// <タイトル>
				//$pre .= "@t " . $s . "\n";
				$title = trim($s);
			}
			if (($s = tagCheck("%%DESC ",$line)) !== null) {
				// <説明文>
				//$pre .= "@d " . $s . "\n";
				$desc = trim($s);
			}
			if (($s = tagCheck("%%BGIMG ",$line)) !== null) {
				// <背景画像のパス>|<オプション>
				$ary = explode("|", $s);
				$post .= "<script>\n";
				$post .= "  let element = document.querySelector(\"body\");\n";
				$post .= "  element.style.background = \"url(" . $ary[0] . ")";
				if ($ary[1]) {
					$post .= " " . $ary[1];
				} 
				$post .= "\";\n";
				$post .= "</script>\n";
			}
			if (($s = tagCheck("%%INCLUDE ",$line)) !== null) {
				$filecontents = file_get_contents($s);
				$ln = array_map('trim', explode("\n", $filecontents));
				md_proc($ln);
			}
			if (($s = tagCheck("%%IMG ",$line)) !== null) {
				// <画像のパス>|<サイズw>|<サイズh>|<alt>|<説明>
				$ary = explode("|", $s);
				if ($ary) {
					if ($ary[4]) {
						$contents .= "<figure>";
					}
					$contents .= "<img class=\"img-fluid\" src=\"" . $ary[0] . "\"";
					if ($ary[1] != "") {
						$contents .= " width=\"{$ary[1]}\"";
					}
					if ($ary[2] != "") {
						$contents .= " height=\"{$ary[2]}\"";
					}
					if ($ary[3] != "") {
						$contents .= " alt=\"{$ary[3]}\"";
					}
					$contents .= "{$onerror}>\n";
					if ($ary[4]) {
						$contents .= "<figcaption>{$ary[4]}</figcaption>\n";
						$contents .= "</figure>";
					}
				}
			}
			if (($s = tagCheck("%%ALERT ",$line)) !== null) {
				// <タイプ> <メッセージ>
				$ary = explode("|", $s);
				if ($ary) {
					$contents .= "<div class=\"alert {$ary[0]}\" role=\"alert\">";
					$contents .= $ary[1];
					if ($ary[2] == "close") {
						$contents .= "<button type=\"button\" class=\"btn-close\" data-bs-dismiss=\"alert\" aria-label=\"Close\"></button>";
					}
					$contents .= "</div>\n";
				}
			}
			if (($s = tagCheck("%%FORM ",$line)) !== null) {
				$ary = explode("|", $s);
				if ($ary) {
					$contents .= "<form name=\"{$ary[0]}\" method=\"post\" action=\"{$ary[1]}\">\n";
				} else {
					$contents .= "<form>\n";
				}
			}
			if (($s = tagCheck("%%FORMEND",$line)) !== null) {
				$contents .= "</form>\n";
			}
			if (($s = tagCheck("%%BLOCK ",$line)) !== null) {
				$ary = explode("|", $s);
				if ($ary) {
					$contents .= "<div";
					if ($ary[0]) {	// class
						$contents .= " class=\"{$ary[0]}\"";
					}
					if ($ary[1]) {	// style
						$contents .= " style=\"{$ary[1]}\"";
					}
					$contents .= ">\n";
				} else {
					$contents .= "<div>\n";
				}
			}
			if (($s = tagCheck("%%BLOCKEND",$line)) !== null) {
				$contents .= "</div>\n";
			}
			if (($s = tagCheck("%%INPUT ",$line)) !== null) {
				$ary = explode("|", $s);
				if ($ary) {
					$contents .= "<input type=\"{$ary[0]}\" name=\"{$ary[1]}\" value=\"{$ary[2]}\">\n";
				}
			}
			if (($s = tagCheck("%%BUTTON ",$line)) !== null) {
				$ary = explode("|", $s);
				if ($ary) {
					$contents .= "<button type=\"{$ary[0]}\" class=\"btn {$ary[1]}\" onClick=\"{$ary[3]}\">";
					$contents .= $ary[2];
					$contents .= "</button>\n";
				}
			}
			if (($s = tagCheck("%%MACRO ",$line)) !== null) {
				/* 読み捨て
				// <名前> <展開内容>
				//%%<マクロ名前>|<オプション>
				$ary = explode("|", trim($s));
				if ($ary) {
					$macro[array_shift($ary)] = implode("|", $ary);
				}
				*/
			}
			if (($s = tagCheck("%%DOWNLOAD ",$line)) !== null) {
				// <ファイル> <ダウンロード時の名称>
				$ary = explode("|", $s);
				if ($ary) {
					if ($ary[1]) {
						$contents .= "<a class=\"download\" href=\"{$ary[0]}\" download=\"{$ary[1]}\">{$ary[1]}</a>\n";
					} else {
						$contents .= "<a class=\"download\" href=\"{$ary[0]}\" download=\"{$ary[0]}\">{$ary[0]}</a>\n";
					}
				}
			}
			// 目次
			if (($s = tagCheck("%%TOC ", $line)) !== null) {
				$idx_no = 1;
				$lvl = 0;
				$n = 0;
				if ($idx) {
					$contents .= "<div class=\"md_toc\"><div class=\"md_toc_title\">" . inline(trim($s)) . "</div>\n";
				}
				foreach ($idx as $v) {
					if ($v["level"] != $lvl) {
						if ($lvl) {
							if ($v["level"] > $lvl) {
								while ($v["level"] > $lvl++) {
									$contents .= "<ol>\n";
									$n++;
								}
								$contents .= "<li><a href=\"#idx{$v["id"]}\">" . inline($v["title"]) . "</a></li>\n";
							} else {
								while ($v["level"] < $lvl--) {
									$contents .= "</ol>\n";
									$n--;
								}
								$contents .= "<li><a href=\"#idx{$v["id"]}\">" . inline($v["title"]) . "</a></li>\n";
							}
						} else {
							$contents .= "<ol>\n";
							$contents .= "<li><a href=\"#idx{$v["id"]}\">" . inline($v["title"]) . "</a></li>\n";
							$n++;
						}
					} else {
						$contents .= "<li><a href=\"#idx{$v["id"]}\">" . inline($v["title"]) . "</a></li>\n";
					}
					$lvl = $v["level"];
				}
				while ($n > 0) {
					$contents .= "</ol>\n";
					$n--;
				}
				if ($idx) {
					$contents .= "</div>\n";
				}
			}
			// 参照文献
			if (($s = tagCheck("%%REF", $line)) !== null) {
				if ($reference) {
					$contents .= "<div class=\"reference\">\n";
					if (trim($s)) {
						$contents .= "<div class=\"reference_title\">" . trim($s) . "</div>";
					}
					$no = 1;
					foreach ($reference as $v) {
						$contents .= "※" . $no++ . "  " . inline($v) . "<br>\n";
					}
					$contents .= "</div>\n";
				}
			}
			// YouTube https://youtu.be/C43pTFyLPDw
			if (($s = tagCheck("%%YOUTUBE", $line)) !== null) {
				$ary = explode("|", trim($s));
				if (count($ary)) {
					$width = 240;
					if ($ary[2]) {	// width
						$width = $ary[2];
					}
					$ary2 = explode("/", $ary[0]);
					$id = $ary2[count($ary2)-1];
					$alt = $ary[1];
					$contents .= "<p><a href=\"http://www.youtube.com/watch?feature=player_embedded&v={$id}\" target=\"_blank\">\n";
					$contents .= "<img src=\"http://img.youtube.com/vi/{$id}/0.jpg\" alt=\"{$alt}\" width=\"{$width}\" border=\"10\"></a></p>\n";
				}
			}
			if (($s = tagCheck("%%ANCHOR", $line)) !== null) {
				$id = trim($s);
				if ($id) {
					$contents .= "<a id=\"{$id}\"></a>\n";
				}
			}
		} else if ($block == 1) {
			// 読み捨て
		} else {
			// Markdown
			// # ## ### #### ##### #####
			if (mb_eregi("^([ 	]*)(\#+) (.*)$", $line, $m)) {
				if ($para) { $para = 0; $contents .= "</p>\n"; }
				$contents .= "<h" . strlen($m[2]) . " id=\"idx{$idx_no}\">" . inline(macro(trim($m[3]))) . "</h" . strlen($m[2]) . ">\n";
				$idx_no++;
				continue;
			}
			// -[]
			if (mb_eregi("-\[([^\]]*)\](.+)", $line, $m)) {
				if ($para) { $para = 0; $contents .= "</p>\n"; }
				if ($m[1]) {
					$contents .= "<input type=\"checkbox\" checked> " . $m[2] . "<br>\n";
				} else {
					$contents .= "<input type=\"checkbox\"> " . inline(macro(trim($m[2]))) . "<br>\n";
				}
				continue;
			}
			// |
			if (mb_eregi("^([ 	]*)\|([^|]+)\|", $line, $m)) {
				// 見出し行
				list($col1, $ary1) = table($line);
				// 2行目(:-のみ)
				if (mb_eregi("^([ 	]*)\|([^|]+)\|", $lines[$lno + 1], $m)) {
					list($col2, $ary2) = table($lines[$lno + 1]);
					if ($col1 == $col2) {	// 項目数一致＝テーブル確定
						if ($para) { $para = 0; $contents .= "</p>\n"; }
						$align = array();
						for ($i = 0; $i < $col2; $i++) {
							if (substr(trim($ary2[$i]), 0, 1) == ":") {
								if (substr(trim($ary2[$i]), strlen(trim($ary2[$i])) - 1) == ":") {
									$align[$i] = "center";
								} else {
									$align[$i] = "left";
								}
							} else if (substr(trim($ary2[$i]), strlen(trim($ary2[$i])) - 1) == ":") {
								$align[$i] = "right";
							} else {
								$align[$i] = "";
							}
						}
						$contents .= "<table class=\"table\">\n<thead>\n<tr>\n";
						for ($i = 0; $i < $col1; $i++) {
							$v = inline(macro(trim($ary1[$i])));
							$contents .= "<th align=\"{$align[$i]}\">{$v}</th>\n";
						}
						$contents .= "</tr>\n</thead>\n<tbody>\n";
						// 残りの行処理
						for ($ln = $lno + 2; $ln < count($lines); $ln++) {
							if (mb_eregi("^([ 	]*)\|([^|]+)\|", $lines[$ln], $m)) {
								list($col, $ary) = table($lines[$ln]);
								$contents .= "<tr>\n";
								for ($i = 0; $i < $col; $i++) {
									$v = inline(macro(trim($ary[$i])));
									$contents .= "<td align=\"{$align[$i]}\">{$v}</td>\n";
								}
								$contents .= "</tr>\n";
								$lc++;
							} else {
								break;
							}
						}
						$lno = $ln;
						$contents .= "</tbody>\n</table>\n";
						continue;
					}
				}
			}
			// *
			if (mb_eregi("^([ 	]*)\*(.*)", $line, $m0)) {
				if (mb_eregi("^([ 	]*)\*(.*)", $lines[$lno + 1], $m)) {
					if ($para) { $para = 0; $contents .= "</p>\n"; }
					$contents .= "<ul>\n";
					$contents .= "<li>{$m0[2]}</li>\n";
					$lno++;
					while (mb_eregi("^([ 	]*)\*(.*)", $lines[$lno], $m)) {
						$v = inline(macro(trim($m[2])));
						$contents .= "<li>{$v}</li>\n";
						$lno++;
					}
					$contents .= "</ul>\n";
					continue;
				}
			}
			// 1. 2. (⋅⋅⋅で子要素は未対応)
			if (mb_eregi("^([ 	]*)([0-9]+)\. (.*)", $line, $m0)) {
				if (mb_eregi("^([ 	]*)([0-9]+)\.(.*)", $lines[$lno + 1], $m)) {
					if ($para) { $para = 0; $contents .= "</p>\n"; }
					$contents .= "<ol>\n";
					$contents .= "<li>{$m0[3]}</li>\n";
					$lno++;
					while (mb_eregi("^([ 	]*)([0-9]+)\.(.*)", $lines[$lno], $m)) {
						$v = inline(macro(trim($m[3])));
						$contents .= "<li>{$v}</li>\n";
						$lno++;
					}
					$contents .= "</ol>\n";
					continue;
				}
			}
			// ```
			if (mb_eregi("^```", $line, $m)) {
				$cnt = 0;
				for ($i = $lno + 1; $i < count($lines); $i++) {
					if (mb_eregi("^```", $lines[$i])) {
						break;
					}
					$cnt++;
				}
				if ($cnt) {		// 終わりが存在
					if ($para) { $para = 0; $contents .= "</p>\n"; }
					$contents .= "<pre>\n";
					for ($i = $lno + 1; $i < count($lines); $i++) {
						if (mb_eregi("^```", $lines[$i], $m)) {
							break;
						}
						$contents .= htmlspecialchars($lines[$i]) . "\n";
					}
					$lno = $i + 1;
					$contents .= "</pre>\n";
					continue;
				}
			}
			// ~~~（クリップボード機能付き）
			if (mb_eregi("^~~~(.*)", $line, $m)) {
				$cnt = 0;
				for ($i = $lno + 1; $i < count($lines); $i++) {
					if (mb_eregi("^~~~", $lines[$i])) {
						break;
					}
					$cnt++;
				}
				if ($cnt) {		// 終わりが存在
					if ($para) { $para = 0; $contents .= "</p>\n"; }
					if (trim($m[1]) == "clipboard") {
						$n = $cnt;
						if ($n > 10) $n = 10;
						$contents .= "<div class=\"clipboard\">\n";
						$contents .= "<textarea id=\"clip{$clipboard}\" class=\"form-control\" rows=\"{$n}\" readonly>\n";
						for ($i = $lno + 1; $i < count($lines); $i++) {
							if (mb_eregi("^~~~", $lines[$i], $m)) {
								break;
							}
							//$contents .= $lines[$i];
							if ($i > ($lno + 1)) {
								$contents .= "\n";
							}
							$contents .= htmlspecialchars($lines[$i]);
						}
						$contents .= "</textarea>\n";
						$contents .= "<a href=\"javascript:clipboard('clip{$clipboard}')\"><svg xmlns=\"http://www.w3.org/2000/svg\" width=\"24\" height=\"24\" viewBox=\"0 0 24 24\" fill=\"none\" stroke=\"currentColor\" stroke-width=\"2\" stroke-linecap=\"round\" stroke-linejoin=\"round\" class=\"feather feather-clipboard\"><path d=\"M16 4h2a2 2 0 0 1 2 2v14a2 2 0 0 1-2 2H6a2 2 0 0 1-2-2V6a2 2 0 0 1 2-2h2\"></path><rect x=\"8\" y=\"2\" width=\"8\" height=\"4\" rx=\"1\" ry=\"1\"></rect></svg> クリップボードにコピー</a>";
						$contents .= "</div>\n";
						$clipboard++;
					} else {
						$contents .= "<pre>\n";
						for ($i = $lno + 1; $i < count($lines); $i++) {
							if (mb_eregi("^~~~", $lines[$i], $m)) {
								break;
							}
							//$contents .= $lines[$i];
							$contents .= htmlspecialchars($lines[$i]) . "\n";
						}
						$contents .= "</pre>\n";
					}
					$lno = $i + 1;
					continue;
				}
			}
			// ___（3つ以上）[class]
			if (mb_eregi("^\_\_[\_]+($|\[([^\]]+)\])", $line, $m)) {
			if ($para) { $para = 0; $contents .= "</p>\n"; }
				if ($m[2]) {
					$contents .= "<hr class=\"{$m[2]}\">\n";
				} else {
					$contents .= "<hr>\n";
				}
				continue;
			}
			// ---（3つ以上）
			if (mb_eregi("^\-\-[\-]+($|\[([^\]]+)\])", $line, $m)) {
				if ($para) { $para = 0; $contents .= "</p>\n"; }
				if ($m[2]) {
					$contents .= "<hr class=\"{$m[2]}\">\n";
				} else {
					$contents .= "<hr>\n";
				}
				continue;
			}
			// ===（3つ以上）
			if (mb_eregi("^\=\=[\=]+($|\[([^\]]+)\])", $line, $m)) {
				if ($para) { $para = 0; $contents .= "</p>\n"; }
				if ($m[2]) {
					$contents .= "<hr class=\"{$m[2]}\">\n";
				} else {
					$contents .= "<hr>\n";
				}
				continue;
			}
			// ***
			if (mb_eregi("^\*\*[\*]+($|\[([^\]]+)\])", $line, $m)) {
				if ($para) { $para = 0; $contents .= "</p>\n"; }
				if ($m[2]) {
					$contents .= "<hr class=\"{$m[2]}\">\n";
				} else {
					$contents .= "<hr>\n";
				}
				continue;
			}
			// >
			if (mb_eregi("^([\> ]+)(.*)$", $line, $m)) {
				if ($para) { $para = 0; $contents .= "</p>\n"; }
				$nl = 0;
				$qn = strlen(str_replace(" ", "", $m[1]));
				$v = inline(macro(trim($m[2])));
				while ($nl < $qn) {
					$contents .= "<blockquote>";
					$nl++;
				}
				$contents .= "{$v}\n";
				while (mb_eregi("^([\> ]+)(.*)$", $lines[$lno + 1], $m)) {
					$qn = strlen(str_replace(" ", "", $m[1]));
					while ($nl < $qn) {
						$contents .= "<blockquote>";
						$nl++;
					}
					while ($nl > $qn) {
						$contents .= "</blockquote>";
						$nl--;
					}
					$v = inline(macro(trim($m[2])));
					$contents .= "{$v}\n";
					$lno++;
				}
				while ($nl > 0) {
					$contents .= "</blockquote>\n";
					$nl--;
				}
				continue;
			}
			// * - + (行頭のみ)
			if (mb_eregi("^([\*\+\-]) (.*)$", $line, $m0)) {
				// 2行以上連続する場合のみ
				if (mb_eregi("^([\*\+\-]) (.*)$", $lines[$lno + 1], $m)) {
					if ($para) { $para = 0; $contents .= "</p>\n"; }
					$contents .= "<ul>\n";
					$contents .= "<li>{$m0[2]}</li>\n";
					$lno++;
					while (mb_eregi("^([\*\+\-]) (.*)$", $lines[$lno], $m)) {
						$v = inline(macro(trim($m[2])));
						$contents .= "<li>{$v}</li>\n";
						$lno++;
					}
					$contents .= "</ul>\n";
					continue;
				}
			}
			// <pre>の間は</pre>まで何もしない
			if (mb_eregi("[ 	]*<pre(.*)", $line, $m)) {
				// 対応する終わりを見つける
				$cnt = 0;
				for ($i = $lno; $i < count($lines); $i++) {
					if (mb_eregi("\<\/pre\>", $lines[$i], $m)) {
						$cnt++;
						break;
					}
				}
				if ($cnt) {
					if ($para) { $para = 0; $contents .= "</p>\n"; }
					// 閉じるまでそのまま出力
					for ($i = $lno; $i < count($lines); $i++) {
						$contents .= trim($lines[$i], "\r\n") . "\n";
						if (mb_eregi("\<\/pre\>", $lines[$i], $m)) {
							break;
						}
					}
					$lno = $i;
					continue;
				}
			}
			// <div>の間は</div>まで何もしない
			if (mb_eregi("([ 	]*)<div(.*)", $line, $m)) {
				// 対応する終わりを見つける
				$cnt = 0;
				for ($i = $lno; $i < count($lines); $i++) {
					if (mb_eregi("</div>", $lines[$i], $m)) {
						$cnt++;
						break;
					}
				}
				if ($cnt) {
					if ($para) { $para = 0; $contents .= "</p>\n"; }
					// 閉じるまでそのまま出力
					for ($i = $lno; $i < count($lines); $i++) {
						$contents .= trim($lines[$i], "\r\n") . "\n";
						if (mb_eregi("</div>", $lines[$i], $m)) {
							break;
						}
					}
					$lno = $i;
					continue;
				}
			}
			// 空行
			if (mb_eregi("^([ 	]*)$", $line, $m)) {
				if ($para) { $para = 0; $contents .= "</p>\n"; }
				continue;
			}
			// 次行のチェック
			// ---
			if (mb_eregi("^([ 	]*)(\-\-[\-]+)$", $lines[$lno + 1], $m)) {
				if ($para) { $para = 0; $contents .= "</p>\n"; }
				$contents .= "<h2>{$line}</h2>\n";
				$lno += 1;
				continue;
			}
			// ===
			if (mb_eregi("^([ 	]*)(\=\=[\=]+)$", $lines[$lno + 1], $m)) {
				if ($para) { $para = 0; $contents .= "</p>\n"; }
				$contents .= "<h1>{$line}</h1>\n";
				$lno += 1;
				continue;
			}
			// [ ... ]: 参照定義は読み捨て
			if (mb_eregi("\[([^\]]*)\]: ([^ ]+)", $line, $m)) {
				continue;
			}
			// パラグラフ
			if ($para) {
				//
			} else {
				$para = 1;
				$contents .= "<p>";
			}
			// 行終わりが空白2つなら改行する
			if (mb_eregi("(.*)  $", $line, $m)) {
				$line = $m[1] . "<br>";
			}
			//if (substr($line, strlen($line) - 2) == "  ") {
			//	$contents .= "<br>";
			//}
			$contents .= inline(macro($line));
 			$contents .= "\n";
 		}
	}
	if ($para) {
		$contents .= "</p>";
		$para = 0;
	}
}
// 拡張マークダウン終わり
